@@ -12,12 +12,13 @@
 /* by Arnaud Le Hors, lehors@sophia.inria.fr,       */
 /* Copyright 1990-93 GROUPE BULL                    */
 
-/* This is Xteddy version 2.0.2 as of 2003-06-30.   */
-/* from Andreas Tille <tille@debian.org>            */
-/* Changes: use Imlib instead of libxpm and load    *
- *          different image formats                 */
+/* This is Xteddy version 2.1 as of 2005-05-25.     */
+/* from  Richard Neill <rn214@hermes.cam.ac.uk>     */
+/* Changes: see ChangeLog                           */
 
 #define DEFAULT_IMAGE_DIR "/usr/share/xteddy"
+#define DEFAULT_LOCAL_IMAGE_DIR "/usr/local/share/xteddy"
+#define XTEDDY_VERSION "2.02"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -48,6 +49,9 @@ static char *progname;
 Display *display;
 int screen_num;
 
+ino_t xteddy_inodenum;	/*  the inode of the executable. */
+struct stat stat_buf_xteddy;
+
 char *Extensions[] = {
   "xpm",
   "png",
@@ -72,10 +76,21 @@ char *InitTeddy(char *teddy)
   char     fbuf[MAXPATHLEN], sbuf[MAXPATHLEN], *name, *ext, *sext;
   struct   stat stat_buf;
   char   **ptr;
+  ino_t   teddy_inodenum;
 
   if ( !teddy || !*(teddy) ) return NULL;
+  if ( !stat(teddy, &stat_buf) ){
+    /* If the executable is in the current directory, xteddy will try to load itself (the binary!) and fail. Don't do this. */
+    teddy_inodenum=stat_buf.st_ino;
+    if (teddy_inodenum==xteddy_inodenum){
+      fprintf(stderr, "Warning: the file named '%s' in the current directory is the executable program itself! Not trying to display it as an image.\n", teddy);
+      /* Note, if the current dir contains a subdir named xteddy and xteddy is invoked as "xteddy" without the path, this error will also be triggered. The error message above
+      will be (misleadingly) printed, but the correct action (not trying to load a directory as an image) will occur. */
+    }else{
+      return strdup(teddy);
+    }
+  }
 
-  if ( !stat(teddy, &stat_buf) ) return strdup(teddy);
   /* Test, some image formats PIXMAP_PATH */
   strcat(strcpy(fbuf, PIXMAP_PATH), "/");
   strcat(strcat(strcpy(sbuf, DEFAULT_IMAGE_DIR), "/"), teddy);
@@ -90,10 +105,24 @@ char *InitTeddy(char *teddy)
     strcat(name, *ptr);
     if ( !stat(name, &stat_buf) ) return strdup(name);
     if ( !stat(fbuf, &stat_buf) ) return strdup(fbuf);
+    strcat(sbuf, ".");   /* missing the dot before the extension! */
     strcat(sbuf, *ptr);
     if ( !stat(sbuf, &stat_buf) ) return strdup(sbuf);
     *ext = *sext = 0;
   }
+
+  /* Repeat with a different search path. */
+  strcat(strcat(strcpy(sbuf, DEFAULT_LOCAL_IMAGE_DIR), "/"), teddy);
+  if ( !stat(sbuf, &stat_buf) ) return strdup(sbuf);
+  sext = sbuf + strlen(sbuf);
+  for ( ptr = Extensions; *ptr; ptr++ ) {
+    strcat(sbuf, ".");   /* missing the dot before the extension! */
+    strcat(sbuf, *ptr);
+    if ( !stat(sbuf, &stat_buf) ) return strdup(sbuf);
+    *sext = 0;
+  }
+
+  /* failed to find anything. */
   return NULL;
 }
 
@@ -109,7 +138,7 @@ int main(int argc, char **argv)
   XSizeHints           size_hints;
   XWMHints             wm_hints;
   XClassHint           class_hints;
-  XTextProperty        windowName, iconName;
+  XTextProperty        windowName;
   int                  argnum;
   int                  use_wm, float_up, allow_quit;
   XEvent               report;
@@ -137,11 +166,14 @@ int main(int argc, char **argv)
     progname = argv[0];
   else
     progname++;
-  teddy = progname; 
+  teddy = progname;
+
 
   /* Option handling: "-wm", "-float", "-noquit", "-geometry" */
   /* and "-display" are recognized. See manual page for details. */
   /* -F<name> ... Other pixmap name */
+  /* -F or -f are equivalent. A space may be included if desired after the option (eg "-f xduck") */
+  /* -v prints version. -h or --help prints help */
   use_wm = FALSE;
   float_up = FALSE;
   allow_quit = TRUE;
@@ -154,13 +186,41 @@ int main(int argc, char **argv)
       float_up = TRUE;
     if (!strcmp(argv[argnum],"-noquit"))
       allow_quit = FALSE;
-    if (!strcmp(argv[argnum],"-geometry")) 
+    if (!strcmp(argv[argnum],"-geometry"))
       geomflags = XParseGeometry(argv[++argnum], &x, &y, &xw, &xh);
     if (!strcmp(argv[argnum],"-display"))
       display_name = argv[++argnum];
-    if (!strncmp(argv[argnum],"-F", 2)) 
+    /* Use -f or -F; -f seems more natural. But -float does not mean "-f loat" ! */
+    if ( (!strncmp(argv[argnum],"-F", 2)) || ( (!strncmp(argv[argnum],"-f", 2)) && (strcmp(argv[argnum],"-float")) ) )
       teddy = argv[argnum] + 2;
+      if (strlen(teddy) == 0){	/* accept "-f <name>" as well as "-f<name>" */
+        teddy = argv[argnum] + 3;
+        argnum ++;
+      }
+    if (!strcmp(argv[argnum],"-v")){
+     fprintf(stderr,"xteddy version: %s\n",XTEDDY_VERSION);
+     exit (0);
+    }
+    if ((!strncmp(argv[argnum],"-h", 2)) || (!strncmp(argv[argnum],"--help", 2))){  /* show help */
+      fprintf (stderr,"xteddy is a cute teddy bear that sits on your X desktop.\n");
+      fprintf (stderr,"Usage: xteddy [ OPTIONS ] [ -f <filename> ]\n");
+      fprintf (stderr,"        -wm                     use the window manager\n");
+      fprintf (stderr,"        -float                  always float on top of other windows\n");
+      fprintf (stderr,"        -noquit                 disable the 'q' key to quit.\n");
+      fprintf (stderr,"        -f,-F <filename>        use <filename> as the image.\n");
+      fprintf (stderr,"        -geometry,-display      have their usual meaning in X.\n");
+      fprintf (stderr,"        -h, --help              display this help and exit\n");
+      fprintf (stderr,"        -v                      print version information and exit\n\n");
+      fprintf (stderr,"The default image is a teddy bear, but others may be selected with -f (or -F).\n");
+      fprintf (stderr,"To tuck him back up in bed, move the mouse over him and press the 'q' or 'Esc' key.\n");
+      fprintf (stderr,"This is xteddy version %s. See `man 1 xteddy` for more information.\n",XTEDDY_VERSION);
+      exit (0);
+    }
   }
+
+  stat(argv[0], &stat_buf_xteddy);  /* find the inode number of the executable file itself */
+  xteddy_inodenum=stat_buf_xteddy.st_ino; /* [This can be fooled if it is invoked as "xteddy" without the path, AND eg the working directory contains a subdirectory called xteddy] */
+
   /* Connect to X server */
   if ( (display = XOpenDisplay(display_name)) == NULL )
     {
@@ -174,11 +234,13 @@ int main(int argc, char **argv)
 
   if ( !(id=Imlib_init(display)) ) return -1;
   if ( !(file = InitTeddy(teddy)) ) {
-    fprintf(stderr, "Can not find any image with name %s.\n", teddy);
+    fprintf(stderr, "Can not find any image with name '%s'.\n", teddy);
+    return -1;
+  }
+  if ( !(im=Imlib_load_image(id, file)) ) {
+    fprintf(stderr, "Most probably, the file '%s' is not a valid image.\n", teddy);
     return -1;
   }
-  if ( !(im=Imlib_load_image(id, file)) ) return -1;
-
   /* Set the window position according to user preferences */
   if (geomflags & XNegative)
     x = display_width - im->rgb_width + x;
@@ -217,25 +279,23 @@ int main(int argc, char **argv)
   size_hints.flags = PPosition | PSize | PMinSize | PMaxSize;
   if (XStringListToTextProperty(&(teddy), 1, &windowName) == 0)
     {
-      (void) fprintf(stderr,
-		     "%s: structure allocation for windowName failed.\n",
-		     progname);
+      (void) fprintf(stderr, "%s: structure allocation for windowName failed.\n", progname);
       Imlib_kill_image(id, im);
       return -1;
     }
   wm_hints.initial_state = NormalState;
   wm_hints.input = TRUE;
   wm_hints.flags = StateHint | IconPixmapHint | InputHint;
-  
+
   class_hints.res_name = progname;
   class_hints.res_class = "Xteddy";
-  
-  XSetWMProperties(display, win, &windowName, &iconName,
+
+  XSetWMProperties(display, win, &windowName, NULL,
 		   argv, argc, &size_hints, &wm_hints, &class_hints);
 
   /* Select event types wanted */
-  inputmask = ExposureMask | KeyPressMask | ButtonPressMask | 
-    ButtonReleaseMask | StructureNotifyMask | ButtonMotionMask | 
+  inputmask = ExposureMask | KeyPressMask | ButtonPressMask |
+    ButtonReleaseMask | StructureNotifyMask | ButtonMotionMask |
       PointerMotionHintMask | EnterWindowMask | LeaveWindowMask;
   if (float_up) inputmask |= VisibilityChangeMask;
   XSelectInput(display, win, inputmask);
@@ -315,10 +375,10 @@ int main(int argc, char **argv)
 	      XRaiseWindow(display,win);
 	  break;
 	case KeyPress:
-	  /* Exit on "q" or "Q" */
+	  /* Exit on "q" or "Q" or "Esc" */
 	  charcount = XLookupString(&report.xkey, buffer, bufsize,
 				    &keysym, &compose);
-	  if((keysym == XK_Q) || (keysym == XK_q))
+	  if((keysym == XK_Q) || (keysym == XK_q)  || (keysym == XK_Escape))
 	    {
               if (allow_quit)
 		{
@@ -335,30 +395,3 @@ int main(int argc, char **argv)
     } /* end while */
 }
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
